
class PlayListBase:
	def __init__(self, player, url, params, curpos):
		self.dataloader = RelatedLoader
	def first(self):
		pass

	def next(self):
		pass

	def previous(self):
		pass

	def last(self):
		pass
	
	def getSource(self):
		pass


